@@ -1,19 +1,7 @@
 /* This file is part of FSView.
-   Copyright (C) 2002, 2003 Josef Weidendorfer <Josef.Weidendorfer@gmx.de>
+    SPDX-FileCopyrightText: 2002, 2003 Josef Weidendorfer <Josef.Weidendorfer@gmx.de>
 
-   KCachegrind is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public
-   License as published by the Free Software Foundation, version 2.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: GPL-2.0-only
 */
 
 /*
@@ -134,11 +122,13 @@ protected:
     bool closeUrl() override;
 
 private:
+    void setNonStandardActionEnabled(const char *actionName, bool enabled);
+    KFileItemList selectedFileItems() const;
+
     FSView *_view;
     FSJob *_job;
     FSViewBrowserExtension *_ext;
     KActionMenu *_visMenu, *_areaMenu, *_depthMenu, *_colorMenu;
-    void setNonStandardActionEnabled(const char *actionName, bool enabled);
 };
 
 #endif // FSVIEW_PART_H
