@@ -16,7 +16,7 @@
 #include "webenginepartdownloadmanager.h"
 #include "webenginewallet.h"
 #include <webenginepart_debug.h>
-#include "webenginepartcertificateerrordlg.h"
+#include "webenginepartcontrols.h"
 
 #include <QWebEngineCertificateError>
 #include <QWebEngineSettings>
@@ -39,6 +39,8 @@
 #include <KUserTimestamp>
 #include <KPasswdServerClient>
 #include <KParts/BrowserInterface>
+#include <KJobWidgets>
+#include <KPluginMetaData>
 
 #include <QStandardPaths>
 #include <QDesktopWidget>
@@ -63,8 +65,7 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
          m_kioErrorCode(0),
          m_ignoreError(false),
          m_part(part),
-         m_passwdServerClient(new KPasswdServerClient),
-         m_wallet(nullptr)
+         m_passwdServerClient(new KPasswdServerClient)
 {
     if (view())
         WebEngineSettings::self()->computeFontSizes(view()->logicalDpiY());
@@ -98,8 +99,6 @@ WebEnginePage::WebEnginePage(WebEnginePart *part, QWidget *parent)
     }
 
     m_part->downloadManager()->addPage(this);
-
-    m_wallet = new WebEngineWallet(this, parent ? parent->window()->winId() : 0);
 }
 
 WebEnginePage::~WebEnginePage()
@@ -138,8 +137,9 @@ static void checkForDownloadManager(QWidget* widget, QString& cmd)
     cmd = exeName;
 }
 
-void WebEnginePage::download(const QUrl& url, const QString& mimetype, bool newWindow)
+void WebEnginePage::download(QWebEngineDownloadItem *it, bool newWindow)
 {
+    QUrl url = it->url();
     // Integration with a download manager...
     if (!url.isLocalFile()) {
         QString managerExe;
@@ -155,8 +155,8 @@ void WebEnginePage::download(const QUrl& url, const QString& mimetype, bool newW
     KParts::BrowserArguments bArgs;
     bArgs.setForcesNewWindow(newWindow);
     KParts::OpenUrlArguments urlArgs;
-    urlArgs.setMimeType(mimetype);
-    emit part()->browserExtension()->openUrlRequest(url, urlArgs, bArgs);
+    urlArgs.metaData().insert("SuggestedFileName", it->suggestedFileName());
+    askBrowserToOpenUrl(url, it->mimeType(), urlArgs, bArgs);
 }
 
 void WebEnginePage::requestOpenFileAsTemporary(const QUrl& url, const QString &mimeType, bool newWindow)
@@ -208,20 +208,37 @@ static bool domainSchemeMatch(const QUrl& u1, const QUrl& u2)
     return (u1List == u2List);
 }
 
-bool WebEnginePage::acceptNavigationRequest(const QUrl& url, NavigationType type, bool isMainFrame)
+bool WebEnginePage::askBrowserToOpenUrl(const QUrl& url, const QString& mimetype, const KParts::OpenUrlArguments &_args, const KParts::BrowserArguments &bargs)
 {
-    if (m_urlRequestedByApp != url) {
-        m_urlRequestedByApp = QUrl();
-        
-        //Don't open local files using WebEnginePart except if configured to do so by the user. For example
-        //for example, this ensures that the "Home" link in the introduction page is opened in Dolphin part 
-        //(or whichever part the user has chosen to open directories instead of WebEnginePart
-        if (url.isLocalFile()) {
-            emit m_part->browserExtension()->openUrlRequest(url);
-            return false;
+    KParts::OpenUrlArguments args(_args);
+    args.setMimeType(mimetype);
+    args.metaData().insert("DontSendToDefaultHTMLPart", "");
+    emit m_part->browserExtension()->openUrlRequest(url, args, bargs);
+    return true;
+}
+
+bool WebEnginePage::shouldOpenLocalUrl(const QUrl& url) const
+{
+    Q_ASSERT(url.isLocalFile());
+    KParts::BrowserInterface *bi = m_part->browserExtension()->browserInterface();
+    if (bi) {
+        QString partToUse;
+        bool success = success = QMetaObject::invokeMethod(bi, "partForLocalFile", Q_RETURN_ARG(QString, partToUse), Q_ARG(QString, url.path()));
+        if (success) {
+            return partToUse == m_part->metaData().name();
         }
     }
-//     qCDebug(WEBENGINEPART_LOG) << url << "type=" << type;
+    return false;
+}
+
+bool WebEnginePage::acceptNavigationRequest(const QUrl& url, NavigationType type, bool isMainFrame)
+{
+    if (isMainFrame && url.isLocalFile()) {
+        if (!shouldOpenLocalUrl(url)) {
+            return askBrowserToOpenUrl(url);
+        }
+    }
+
     QUrl reqUrl(url);
 
     // Handle "mailto:" url here...
@@ -245,8 +262,8 @@ bool WebEnginePage::acceptNavigationRequest(const QUrl& url, NavigationType type
         case QWebEnginePage::NavigationTypeFormSubmitted:
             if (!checkFormData(url))
                return false;
-            if (m_wallet) {
-                m_wallet->saveFormsInPage(this);
+            if (part() && part()->wallet()) {
+                part()->wallet()->saveFormsInPage(this);
             }
 
             break;
@@ -348,47 +365,9 @@ static int errorCodeFromReply(QNetworkReply* reply)
 }
 #endif
 
-bool WebEnginePage::certificateError(const QWebEngineCertificateError& _ce)
+bool WebEnginePage::certificateError(const QWebEngineCertificateError& ce)
 {
-    QWebEngineCertificateError ce(_ce);
-    if (!ce.isOverridable()) {
-        return false;
-    }
-    int error = static_cast<int>(ce.error());
-    QString url = ce.url().url();
-    KConfigGroup grp(KSharedConfig::openConfig(), "CertificateExceptions");
-    QList<int> exceptionsForUrl = grp.readEntry(url, QList<int>{});
-    if (exceptionsForUrl.contains(error)) {
-        return true;
-    } else {
-        ce.defer();
-        WebEnginePartCertificateErrorDlg *dlg = new WebEnginePartCertificateErrorDlg(ce, view());
-        connect(dlg, &WebEnginePartCertificateErrorDlg::finished, this, [this, dlg](int _){handleCertificateError(dlg);});
-        dlg->open();
-        return true;
-    }
-}
-
-void WebEnginePage::handleCertificateError(WebEnginePartCertificateErrorDlg *dlg) {
-    if (!dlg) {
-        return;
-    }
-    QWebEngineCertificateError error = dlg->certificateError();
-    WebEnginePartCertificateErrorDlg::UserChoice choice = dlg->userChoice();
-    dlg->deleteLater();
-    if (choice == WebEnginePartCertificateErrorDlg::UserChoice::DontIgnoreError) {
-        error.rejectCertificate();
-    } else {
-        error.ignoreCertificateError();
-        if (choice == WebEnginePartCertificateErrorDlg::UserChoice::IgnoreErrorForever) {
-            KConfigGroup grp(KSharedConfig::openConfig(), "CertificateExceptions");
-            QString url = error.url().url();
-            QList<int> exceptionsForUrl = grp.readEntry(url, QList<int>{});
-            exceptionsForUrl.append(error.error());
-            grp.writeEntry(url, exceptionsForUrl);
-            grp.sync();
-        }
-    }
+    return WebEnginePartControls::self()->handleCertificateError(ce, this);
 }
 
 WebEnginePart* WebEnginePage::part() const
@@ -520,6 +499,8 @@ void WebEnginePage::slotFeaturePermissionRequested(const QUrl& url, QWebEnginePa
     //url.path() is always / (meaning that permissions should be granted site-wide and not per page)
     QUrl thisUrl(this->url());
     thisUrl.setPath("/");
+    thisUrl.setQuery(QString());
+    thisUrl.setFragment(QString());
     if (url == thisUrl) {
         part()->slotShowFeaturePermissionBar(url, feature);
         return;
@@ -917,14 +898,7 @@ bool NewWindowPage::acceptNavigationRequest(const QUrl &url, NavigationType type
             this->deleteLater();
             return false;
         }
-        // Reparent this page to the new webview to prevent memory leaks.
-        setParent(webView);
-        // Replace the webpage of the new webview with this one. Nice trick...
-        webView->setPage(this);
-        // Set the new part as the one this page will use going forward.
-        setPart(webenginePart);
-        // Connect all the signals from this page to the slots in the new part.
-        webenginePart->connectWebEnginePageSignals(this);
+        webenginePart->setPage(this);
         //Set the create new window flag to false...
         m_createNewWindow = false;
 
